# Generated by roxygen2: do not edit by hand

export(FDRCalc)
export(Popinitial_fixknots)
export(cptgaControl)
export(crossover_fixknots)
export(fixknotsIC)
export(gareg_knots)
export(gareg_subset)
export(mutation_fixknots)
export(selectTau_uniform_exact)
export(splineX)
export(subsetBIC)
export(varyknotsIC)
exportClasses(gareg)
exportMethods(show)
exportMethods(summary)
importClassesFrom(GA,ga)
importClassesFrom(GA,gaisl)
importClassesFrom(changepointGA,cptga)
importClassesFrom(changepointGA,cptgaisl)
importFrom(GA,ga)
importFrom(GA,gaisl)
importFrom(changepointGA,cptga)
importFrom(changepointGA,cptgaisl)
importFrom(methods,new)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(stats,.lm.fit)
importFrom(stats,glm.control)
importFrom(stats,glm.fit)
importFrom(stats,lm.fit)
importFrom(stats,runif)
importFrom(utils,str)
