#' @importFrom changepointGA cptga cptgaisl
#' @importClassesFrom changepointGA cptga cptgaisl
#' @importFrom GA ga gaisl
#' @importClassesFrom GA ga gaisl
#' @importFrom methods show setGeneric setMethod slot slotNames new
#' @importFrom stats runif lm.fit
#' @importFrom utils str
NULL
