% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gareg_knots.R
\docType{data}
\name{.cptgaisl.default}
\alias{.cptgaisl.default}
\title{Default Controls for \code{cptgaisl} (Island GA)}
\format{
A named list with fields like \code{popSize}, \code{numIslands},
\code{pcrossover}, \code{pmutation}, \code{maxMig}, \code{maxgen}, etc.
}
\usage{
.cptgaisl.default
}
\description{
Engine defaults used by \link{cptgaControl} when \code{engine = "cptgaisl"}.
Includes island-specific fields (e.g., \code{numIslands}, \code{maxMig}).
}
\seealso{
\link{cptgaControl}, \link{.cptga.default}
}
\keyword{internal}
